//
//  AgoraGameEngine.h
//  AgoraGameEngineKit
//
//  Created by dingyusong on 2022/3/11.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import "AMCEnumerates.h"

NS_ASSUME_NONNULL_BEGIN


@interface AGELogConfig : NSObject

/// log文件路径，你需要保证路径的合法性,默认沙盒Documents/AgoraGame.log
@property (nonatomic, copy) NSString *filePath;

/// log文件大小单位KB，默认1024KB
@property (nonatomic, assign) NSInteger fileSizeInKB;

/// log level, 默认是AGELogLevelInfo
@property (nonatomic, assign) AGELogLevel level;

- (instancetype)initWithFilePath:(NSString *)filePath fileSizeInKB:(NSInteger)fileSizeInKB level:(AGELogLevel)level;
- (instancetype)initWithFilePath:(NSString *)filePath fileSizeInKB:(NSInteger)fileSizeInKB;
- (instancetype)initWithFilePath:(NSString *)filePath ;
- (instancetype)init;
@end

typedef NSString *AGEOptionID NS_STRING_ENUM;

FOUNDATION_EXPORT AGEOptionID const AGEOptionIDGetGameList;
FOUNDATION_EXPORT AGEOptionID const AGEOptionIDGameState;
FOUNDATION_EXPORT AGEOptionID const AGEOptionIDSendGift;

@protocol AGEGameEngineEventProtocol <NSObject>
- (void)onSetOptionResult:(NSInteger)operationId optionId:(NSString *)optionId result:(BOOL)result reason:(NSString * _Nullable)reason;
- (void)onGetOptionResult:(NSInteger)operationId optionId:(NSString *)optionId result:(BOOL)result outOption:(NSString * _Nullable)outOption;
- (void)onMessage:(NSString *)messageId message:(NSString *)message;
@end

@interface AGEGameContext : NSObject
@property (nonatomic, copy) NSString *appId;
@property (nonatomic, copy) NSString *uid;
@property (nonatomic, copy) NSString *rtmToken;
@property (nonatomic, weak) id<AGEGameEngineEventProtocol> delegate;
@property (nonatomic, strong) AGELogConfig *logconfig;
@end

@interface AGEGameOptions : NSObject
@property (nonatomic, copy) NSString *gameId;
@property (nonatomic, copy) NSString *roomId;
@property (nonatomic, copy) NSString *userId;
@property (nonatomic, copy) NSString *role;
@property (nonatomic, copy) NSString *language;
@property (nonatomic, copy) NSString *options;
@end

@interface AgoraGameEngine : NSObject

/// AgoraGameEngine单例
+ (instancetype)sharedInstance;

/// 获取版本号
+ (NSString *)getVersion;

/// 设置engineContext
/// @param gameContext engine的必要参数
- (AGEErrorCode)setupEngineWithGameContext:(AGEGameContext *)gameContext NS_SWIFT_NAME(setup(gameContext:));

/// 更新token
/// @param token 游戏相关信息，注意gameContext.rtmtoken的周期性更新
- (AGEErrorCode)renewToken:(NSString *)token;

/// 加载游戏，如果已经加载了游戏，会销毁上一个游戏的webview，然后新建一个webview放到传的view容器中
/// @param options 游戏相关信息
/// @param view 游戏载入的容器
- (AGEErrorCode)loadGameWithOptions:(AGEGameOptions *)options view:(UIView *)view NS_SWIFT_NAME(loadGame(options:view:));

/// 设置数据
/// @param operationId 操作id，在代理回调中用来标识某个操作
/// @param optionId 操作类别 支持的optionId：AGEOptionIDGameState，AGEOptionIDSendGift，但不仅限于这些，后期可根据业务增加
/// @param args 参数json格式
- (AGEErrorCode)setOptionWithOperationId:(NSInteger )operationId optionId:(NSString *)optionId args:(NSString *)args NS_SWIFT_NAME(setOption(operationId:optionId:args:));

/// 获取数据
/// @param operationId 操作id，在代理回调中用来标识某个操作
/// @param optionId 操作类别 支持的optionId：AGEOptionIDGetGameList，但不仅限于这些，后期可根据业务增加
/// @param args 参数json格式
- (AGEErrorCode)getOptionWithOperationId:(NSInteger )operationId optionId:(NSString *)optionId args:(NSString *)args NS_SWIFT_NAME(getOption(operationId:optionId:args:));

/// 离开游戏，会销毁webview
- (AGEErrorCode)leaveGame;

/// 销毁资源
- (AGEErrorCode)destroy;

@end

NS_ASSUME_NONNULL_END
